'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.alreadyDefined = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = duck;

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var defaultMapper = {
  reducer: function reducer(actionType) {
    return actionType;
  },
  resolve: function resolve(actionType) {
    return actionType + '_RESOLVED';
  },
  reject: function reject(actionType) {
    return actionType + '_REJECTED';
  }
};

var alreadyDefined = exports.alreadyDefined = {};

function checkActionTypeName(actionType, duckName) {
  (0, _invariant2.default)(typeof actionType === 'string', 'Action Type: Expected a string. Got %s instead', actionType);
  if (duckName) {
    (0, _warning2.default)(actionType.split('.')[0] === duckName || actionType.split('/')[0] === duckName, 'Action Type: Expected a string prefixed by \'' + duckName + '\'. Got \'' + actionType + '\' instead');
  }
}

function checkUniqueDefinition(actionType) {
  (0, _warning2.default)(!alreadyDefined[actionType], 'Duplicate definition for Action(' + actionType + ',...');
  if (process.env.NODE_ENV !== 'production') {
    alreadyDefined[actionType] = true;
  }
}

function checkActionObject(obj) {
  (0, _invariant2.default)((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object', 'Action Object: Expected an object. Got %s instead', obj);
  (0, _invariant2.default)(typeof obj.creator === 'function', 'Action creator: Expected a function. Got %s instead', obj.creator);
}

function trackReducers(mapper, actionType, reducerCases, reducers) {
  if (typeof reducerCases === 'function') {
    // shortcut for only one reducer
    (0, _warning2.default)(!reducers[actionType], 'Duplicate reducer case for %s', actionType);
    reducers[actionType] = reducerCases;
  } else if ((typeof reducerCases === 'undefined' ? 'undefined' : _typeof(reducerCases)) === 'object') {
    Object.keys(reducerCases).forEach(function (reducerType) {
      var mapping = mapper[reducerType];
      (0, _warning2.default)(mapping, 'Unknown reducer mapping "%s"', reducerType);
      var t = mapping(actionType);
      (0, _warning2.default)(!reducers[t], 'Duplicate case for "%s"', t);
      reducers[t] = reducerCases[reducerType];
    });
  } else {
    console.error('Wrong type for reducerCases: Expected an object. Got ' + (typeof reducerCases === 'undefined' ? 'undefined' : _typeof(reducerCases)) + 'instead');
  }
}

function duck(duckName, initialState) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$mapper = _ref.mapper,
      mapper = _ref$mapper === undefined ? defaultMapper : _ref$mapper;

  (0, _warning2.default)(typeof duckName === 'string', 'Fist argument of Duck should be a string (name of the Duck)');

  var reducers = {};

  return {
    // new action creators, with corresponding reducers
    defineAction: function defineAction(actionType, reducersAndCreator) {
      checkActionTypeName(actionType, duckName);
      checkUniqueDefinition(actionType);
      checkActionObject(reducersAndCreator);

      var creator = reducersAndCreator.creator,
          reducerCases = _objectWithoutProperties(reducersAndCreator, ['creator']);

      trackReducers(mapper, actionType, reducerCases, reducers);
      return function () {
        var actionObject = creator.apply(undefined, arguments);
        actionObject.type = actionType;
        return actionObject;
      };
    },

    // additional reducer rules (to match actions from other ducks)
    addReducerCase: function addReducerCase(actionType, reducerCases) {
      checkActionTypeName(actionType);
      trackReducers(mapper, actionType, reducerCases, reducers);
    },

    // combined reducer for the duck
    reducer: function reducer() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { type: 'missing action' };

      var transformedState = reducers['*'] ? reducers['*'](state, action) : state;
      return reducers[action.type] ? reducers[action.type](transformedState, action) : transformedState;
    }
  };
}